'use strict';

const Controller = require('egg').Controller;
// APP新增用户总数统计分析模块
class AppNewUserController extends Controller {

  constructor(ctx) {
    super(ctx);
    // 特殊的入参校验可以重写在这，默认可以使用commonRule
    this.createRule = {
      startTime: { type: 'number', min: 0 }, // 开始时间的时间戳  包含
      endTime: { type: 'number', min: 0 }, // 结束时间的时间戳  不包含(客户端处理结束时间  下一日 ，下一月第一日 ，下一年第一日)
    };
  }

  // app新增用户总数
  async index() {
    const { ctx, service } = this;
    // 校验参数 组装参数
    const payload = ctx.validate();
    // 调用 Service 进行业务处理
    if (ctx.isDev()) {
      const data = ctx.getData(payload);
      ctx.success({ data });
    } else {
      const data = await service.tRbacUserService.index(payload, 'newAppTotal');
      // 设置响应内容和响应状态码
      ctx.success({ data });
    }
  }

  // app新增用户位置分布
  async location() {
    const { ctx, service } = this;
    // 校验参数 组装参数
    const payload = ctx.validate(this.createRule);

    // 调用 Service 进行业务处理
    if (ctx.isDev()) {
      const data = ctx.getArea(payload);
      ctx.success({ data });
    } else {
      const data = await service.onlineUserService2.place('appNewLocation', payload);
      // 设置响应内容和响应状态码
      ctx.success({ data });
    }
  }

  // app新增车主性别和年龄
  async sexAndAge() {
    const { ctx, service } = this;

    // 调用 Service 进行业务处理
    if (ctx.isDev()) {
      const data = ctx.getSex2Age();
      ctx.success({ data });
    } else {
      const data = await service.tRbacUserService.sexAndAge('newAppSexAndAgeTotal');
      // 设置响应内容和响应状态码
      ctx.success({ data });
    }
  }

}

module.exports = AppNewUserController;
