'use strict';

const Service = require('egg').Service;

class TRbacUserService extends Service {

  async index(cond, condStr = 'total') {
    const { ctx } = this;
    const maxStrArray = [ 'total', 'appTotal', 'iviTotal' ];
    let agg = [];
    if (maxStrArray.indexOf(condStr) !== -1) {
      agg = ctx.helper.getCommonAggMax({ ...cond, value: condStr });
    } else {
      agg = ctx.helper.getCommonAggSum({ ...cond, value: condStr });
    }
    return await ctx.model.Local.TRbacUserModel.aggregateFix(agg);
  }

  async sexAndAge(condStr = 'sexAndAgeTotal') {
    const { ctx } = this;
    const oneCond = { create_date: 1 };
    oneCond[condStr] = 1;
    const result = await ctx.model.Local.TRbacUserModel.findOne({}, oneCond)
      .sort({ create_date: -1 });
    if (result) {
      return { time: ctx.helper.formatTime1(result.create_date), list: result[condStr] };
    }
    return { time: '', list: [] };
  }

  async statistics({ timeRangData, initData, isForceUpdate }) {
    const { ctx } = this;
    const hasData = await ctx.service.statisticsService.saveBefore(ctx.model.Local.TRbacUserModel,
      { ...initData });
    if (hasData && !isForceUpdate) {
      return;
    }
    initData.start_time = new Date();
    const total = await this.total({ create_date: { $lt: timeRangData.endTime } });
    const sexAndAgeTotal = await this.sex2Age({ create_date: { $lt: timeRangData.endTime } });
    const appTotal = await this.total({
      create_date: { $lt: timeRangData.endTime }, user_orgin: 'APP' });
    const appSexAndAgeTotal = await this.sex2Age({
      create_date: { $lt: timeRangData.endTime }, user_orgin: 'APP' });
    const iviTotal = await this.total({
      create_date: { $lt: timeRangData.endTime }, user_orgin: 'IVI' });
    const iviSexAndAgeTotal = await this.sex2Age({
      create_date: { $lt: timeRangData.endTime }, user_orgin: 'IVI' });
    const newAppTotal = await this.total({
      create_date: { $gte: timeRangData.startTime, $lt: timeRangData.endTime }, user_orgin: 'APP' });

    const newAppSexAndAgeTotal = await this.sex2Age({
      create_date: { $gte: timeRangData.startTime, $lt: timeRangData.endTime }, user_orgin: 'APP' });
    const newIviTotal = await this.total({
      create_date: { $gte: timeRangData.startTime, $lt: timeRangData.endTime }, user_orgin: 'IVI' });

    const newIviSexAndAgeTotal = await this.sex2Age({
      create_date: { $gte: timeRangData.startTime, $lt: timeRangData.endTime }, user_orgin: 'IVI' });
    const activeAppTotal = await this.activeAppTotal(timeRangData);
    const activeAppSexAndAgeTotal = await this.activeAppSex2Age(timeRangData);
    const activeIviTotal = await this.activeIviTotal(timeRangData);
    const activeIviSexAndAgeTotal = await this.activeIviSex2Age(timeRangData);

    // 目前的所有性别年龄统计 和这下面的3个 算法-算车主都是算最终态
    const saledTotal = await this.total({ role_id: ctx.helper.saledRoleId });
    const saledAppTotal = await this.total({ user_orgin: 'APP', role_id: ctx.helper.saledRoleId });
    const saledIviTotal = await this.total({ user_orgin: 'IVI', role_id: ctx.helper.saledRoleId });
    await ctx.service.statisticsService.save(ctx.model.Local.TRbacUserModel,
      { ...initData, total, sexAndAgeTotal,
        appTotal, appSexAndAgeTotal,
        newAppTotal, newAppSexAndAgeTotal,
        activeAppTotal, activeAppSexAndAgeTotal,
        iviTotal, iviSexAndAgeTotal,
        newIviTotal, newIviSexAndAgeTotal,
        activeIviTotal, activeIviSexAndAgeTotal,
        saledTotal, saledAppTotal, saledIviTotal,
      }, isForceUpdate);
  }

  async total(cond) {
    const { ctx } = this;
    return await ctx.model.TRbacUserModel.find(cond).countDocuments();
  }

  async sex2Age(cond) {
    const { ctx } = this;
    const agg = [
      { $match: { ...cond, role_id: ctx.helper.saledRoleId } },
      ...ctx.helper.getAggAndSexMongo(),
    ];
    return await ctx.model.TRbacUserModel.aggregateFix(agg);
  }

  // TODO
  async activeAppTotal({ startTime, endTime }) {
    const { ctx } = this;
    const onlineAgg = [
      { $match: { ...ctx.helper.getTimeRangMatch(startTime, endTime, 'login_time'), login_state: 1 } },
      { $group: {
        _id: '$user_Id',
        login_count: { $sum: 1 },
      } },
      { $match: { login_count: { $gte: 3 } } },
    ];
    const appAgg = [
      { $match: ctx.helper.getTimeRangMatch(startTime, endTime, 'create_time') },
      { $group: { _id: '$user_id', use_duration: { $sum: '$use_duration' } } },
      { $match: { use_duration: { $gt: 30 * 60 * 1000 } } },
    ];
    const result1 = await ctx.model.AppOnlineUserModel.aggregateFix(onlineAgg);
    const result2 = await ctx.model.AppBehaviorRecordModel.aggregateFix(appAgg);
    const map1 = result1.map(item => item._id);
    const map2 = result2.map(item => item._id);
    const unionArray = ctx.helper.unionArray(map1, map2);
    // return unionArray.length;
    return await ctx.model.TRbacUserModel.find({ user_id: { $in: unionArray } }).countDocuments();
  }

  async activeAppSex2Age({ startTime, endTime }) {
    const { ctx } = this;
    const onlineAgg = [
      { $match: { ...ctx.helper.getTimeRangMatch(startTime, endTime, 'login_time'), login_state: 1 } },
      { $group: {
        _id: '$user_Id',
        login_count: { $sum: 1 },
      } },
      { $match: { login_count: { $gte: 3 } } },
    ];
    const appAgg = [
      { $match: ctx.helper.getTimeRangMatch(startTime, endTime, 'create_time') },
      { $group: { _id: '$user_id', use_duration: { $sum: '$use_duration' } } },
      { $match: { use_duration: { $gt: 30 * 60 * 1000 } } },
    ];
    const result1 = await ctx.model.AppOnlineUserModel.aggregateFix(onlineAgg);
    const result2 = await ctx.model.AppBehaviorRecordModel.aggregateFix(appAgg);
    const unionArray = ctx.helper.unionArray(result1.map(item => item._id), result2.map(item => item._id));
    const agg = [
      { $match: { user_id: { $in: unionArray } } },
      ...ctx.helper.getAggAndSexMongo(),
    ];
    return await ctx.model.TRbacUserModel.aggregateFix(agg);
  }

  async activeIviTotal({ startTime, endTime }) {
    const { ctx } = this;
    const agg = [
      { $match: { ...ctx.helper.getTimeRangMatch(startTime, endTime, 'login_time'), login_state: 1 } },
      { $group: {
        _id: '$user_id' },
      },
    ];
    const result = await ctx.model.IviOnlineUserModel.aggregateFix(agg);
    return await ctx.model.TRbacUserModel.find({ user_id: { $in: result.map(item => item._id) } }).countDocuments();
  }

  async activeIviSex2Age({ startTime, endTime }) {
    const { ctx } = this;
    const onlineAgg = [
      { $match: { ...ctx.helper.getTimeRangMatch(startTime, endTime, 'login_time'), login_state: 1 } },
      { $group: {
        _id: '$user_id',
      } },
    ];
    const result = await ctx.model.IviOnlineUserModel.aggregateFix(onlineAgg);
    const unionArray = result.map(item => item._id);
    const agg = [
      { $match: { user_id: { $in: unionArray } } },
      ...ctx.helper.getAggAndSexMongo(),
    ];
    return await ctx.model.TRbacUserModel.aggregateFix(agg);
  }
  // TODO
}

module.exports = TRbacUserService;
