'use strict';

const Service = require('egg').Service;

class TRegisterInfoService extends Service {

  async register(cond) {
    const { ctx } = this;
    const agg = ctx.helper.getCommonAggSum({ ...cond, value: 'total' });
    return await ctx.model.Local.TRegisterInfoModel.aggregateFix(agg);
  }

  // 如果在用户表计算 无法算到过去的实销用户数量
  async saled(cond) {
    const { ctx } = this;
    const agg = ctx.helper.getCommonAggMax({ ...cond, value: 'saledTotal' });
    return await ctx.model.Local.TRegisterInfoModel.aggregateFix(agg);
  }

  async saledExt() {
    const { ctx } = this;
    // 最高哪天  最高是哪月  当前总数   当前总数app   当前总数ivi
    const max = await ctx.model.Local.TRegisterInfoModel.findOne().sort({ total: -1 });
    let maxDay = {};
    if (max) {
      maxDay = { _id: { year: max.year, month: max.month, day: max.day }, count: max.total };
    }
    const agg = [
      { $group: { _id: { year: '$year', month: '$month' }, count: { $sum: '$total' } } },
      { $sort: { count: -1 } },
      { $group: { _id: null, data: { $first: '$$ROOT' } } },
      // 或者
      // { $group: { _id: null, data: { $max: { count: '$count', _id: { year: '$_id.year', month: '$_id.month' } } } } },
    ];
    const result = await ctx.model.Local.TRegisterInfoModel.aggregateNGroup(agg);
    const maxMonth = result.data || {};
    const user = await ctx.model.Local.TRbacUserModel.findOne({}, {
      appTotal: 1, iviTotal: 1,
      saledTotal: 1, saledAppTotal: 1, saledIviTotal: 1,
    }).sort({ create_date: -1 });
    let appTotal = 0;
    let iviTotal = 0;
    let saledTotal = 0;
    let saledAppTotal = 0;
    let saledIviTotal = 0;
    if (user) {
      appTotal = user.appTotal;
      iviTotal = user.iviTotal;
      saledTotal = user.saledTotal;
      saledAppTotal = user.saledAppTotal;
      saledIviTotal = user.saledIviTotal;
    }
    return { maxDay, maxMonth, saledTotal, saledAppTotal, saledIviTotal, appTotal, iviTotal };
  }

  // ====================清洗分割线


  async statistics({ timeRangData, initData, isForceUpdate }) {
    const { ctx } = this;
    const hasData = await ctx.service.statisticsService.saveBefore(ctx.model.Local.TRegisterInfoModel,
      { ...initData });
    if (hasData && !isForceUpdate) {
      return;
    }
    initData.start_time = new Date();
    const total = await this.count(timeRangData);
    const saledTotal = await this.saledCount(timeRangData);

    await ctx.service.statisticsService.save(ctx.model.Local.TRegisterInfoModel,
      { ...initData, total, saledTotal }, isForceUpdate);
  }


  async count({ startTime, endTime }) {
    const { ctx } = this;
    return await ctx.model.TRegisterInfoModel.find(ctx.helper.getTimeRangMatch(startTime,
      endTime, 'update_time')).countDocuments();
  }

  async saledCount({ endTime }) {
    const { ctx } = this;
    const agg = [
      { $match: { is_saled_car: 1, sale_date: { $lt: endTime }, user_id: { $ne: null } } },
      { $group: {
        _id: '$user_id',
      } },
    ];
    const result = await ctx.model.TVehicleRecordModel.aggregateFix(agg);
    return await ctx.model.TRbacUserModel.find({ role_id: ctx.helper.saledRoleId,
      user_id: { $in: result.map(item => item._id) } }).countDocuments();
  }
}

module.exports = TRegisterInfoService;
